#!/usr/bin/python

from __future__ import print_function

from argparse import ArgumentParser
from datetime import (
    datetime,
    timedelta,
)
import re
import sys
from textwrap import wrap

from launchpadlib.launchpad import Launchpad

lp = Launchpad.login_with('release-bot',
                          service_root='https://api.launchpad.net',
                          version='devel')

TEMPLATE = """\
Release checklist for {project}-{milestone}
{milestone_link}

Meeting Attendees: 
Key StakeHolders:  <Juju>, <Conjure-up>, <CDO/OIL>, <Docs>, <CPE>, <Openstack>, <Landscape>,  <JAAS/GUI>, <LXD>, <MAAS>
Date:

Test Results:
CI: {project_ci_link}

CDO: {cdo_qa_link}

Compatibility Matrix:

http://people.canonical.com/~jog/cdo_qa_test_report/last_passed_deployments.html

Main Features of this release:

{features}

Issues Resolved in this release: {num_closed_bugs}

{closed_bugs}

Open Release-Blocking Issues (stable-blocker, rc-blocker): {num_blocker_tag_bugs}

{blocker_tag_bugs}

Open CDO-Blocking Issues (cdo-qa-blocker): {num_cdo_blocker_bugs}

{cdo_blocker_bugs}

Open Critical lssues: {num_critical_bugs}

{critical_bugs}

Other Issues Anyone?

Documentation Ready:

{docs}

Release Notes Available:    

{milestone}: {release_notes}
"""

def _parse_bug_tasks(tasks):
    bugs = []
    for bugtask in tasks:
        bug = bugtask.bug
        
        if bugtask.assignee:
            assignee = bugtask.assignee.display_name
        else:
            assignee = u'Unassigned'

        if bugtask.milestone:
            milestone = bugtask.milestone.name
        else:
            milestone = u'None'
        
        bugs.append((bug.id, bug.date_created, bugtask.status,
                     bugtask.importance, milestone, bug.owner.display_name,
                     assignee, bug.title.capitalize() ))
    return bugs

def get_milestone_bugs(milestone, project, status=None,
                       importance=None, tag=None):
    """Return a list of bug information."""
    lp_project = lp.projects[project]
    lp_milestone = lp_project.getMilestone(name=milestone)

    if status:
        tasks = lp_milestone.searchTasks(status=[status])
    elif importance:
        tasks = lp_milestone.searchTasks(importance=[importance])
    elif tag:
        tasks = lp_milestone.searchTasks(tags=[tag])

    return _parse_bug_tasks(tasks)


def get_project_bugs(milestone, project, status=None,
                     importance=None, tag=None):
    """Return a list of bug information"""
    lp_project = lp.projects[project]

    if status:
        tasks = lp_project.searchTasks(status=[status])
    elif importance:
        tasks = lp_project.searchTasks(importance=[importance])
    elif tag:
        tasks = lp_project.searchTasks(tags=tag)
    
    return _parse_bug_tasks(tasks)


def format_bug_text(bugs):
    """Return the list of bug information as formatted text."""
    resolved = []
    resolved.append(
        u"#\tCreated\t\tStatus\t\t\tImportance\t\t\tMilestone\t\t\tCreator\t\t\tTitle")

    for bug in bugs:
        resolved.append(u"{}\t{}\t{}\t\t{}\t\t\t{}\t\t\t{}\t\t\t{}".format(
            bug[0], bug[1].date(), bug[2], bug[3], bug[4], bug[5], bug[7]))
    resolved_text = '\n'.join(resolved)
    return resolved_text, len(bugs)


def make_notes(milestone, project, revision):
    """Return to formatted release notes."""

    lp_project = lp.projects[project]
    lp_milestone = lp_project.getMilestone(name=milestone)

    milestone_link=lp_milestone.web_link
    features = u'[[Add the features changes here.]]'

    bugs = get_milestone_bugs(milestone, project, 'Fix Committed')
    closed_bugs, num_closed_bugs = format_bug_text(bugs)

    bugs = get_project_bugs(milestone, project,
                            None, None, ['stable-blocker', 'rc-blocker'])
    blocker_tag_bugs, num_blocker_tag_bugs = format_bug_text(bugs)

    bugs = get_project_bugs(milestone, project,
                            None, None, ['cdo-qa-blockers'])
    cdo_blocker_bugs, num_cdo_blocker_bugs = format_bug_text(bugs)

    bugs = get_project_bugs(milestone, project, None, 'Critical')
    critical_bugs, num_critical_bugs = format_bug_text(bugs)
    
    project_ci_link = u'http://qa.jujucharms.com/releases/{}'.format(revision)
    cdo_qa_link = u'http://people.canonical.com/~jog/cdo-qa-test-report/'
    docs = u'[[Add the link to the documentation here.]]'
    release_notes = u'[[Add the link to release notes here.]]'

    text = TEMPLATE.format(
        project=project,
        milestone=milestone,
        milestone_link=milestone_link,
        features=features,
        num_closed_bugs=num_closed_bugs,
        closed_bugs=closed_bugs,
        num_cdo_blocker_bugs=num_cdo_blocker_bugs,
        cdo_blocker_bugs=cdo_blocker_bugs,
        num_blocker_tag_bugs=num_blocker_tag_bugs,
        blocker_tag_bugs=blocker_tag_bugs,
        num_critical_bugs=num_critical_bugs,
        critical_bugs=critical_bugs,
        project_ci_link=project_ci_link,
        cdo_qa_link=cdo_qa_link,
        docs=docs,
        release_notes=release_notes)
    # Normalise the whitespace between sections. The text can have
    # extra whitespae when blank sections are interpolated.
    text = text.replace('\n\n\n\n', '\n\n\n')
    return text


def save_notes(text, file_name):
    """Save the notes to the named file or print to stdout."""
    if file_name is None:
        print(text)
    else:
        with open(file_name, 'w') as rn:
            rn.write(text)


def parse_args(args=None):
    parser = ArgumentParser('Create release checklist for a milestone')
    parser.add_argument(
        '--file-name', help='the name of file to write.', default=None)
    parser.add_argument('project', help='the launchpad project to examine.')
    parser.add_argument('milestone', help='the milestone to examine.')
    parser.add_argument('revision', help='the CI revision to examine.')
    return parser.parse_args(args)


def main(argv):
    args = parse_args(argv[1:])
    print('Generating notes for {} based on milestone {}, revision {}'.format(
        args.project, args.milestone, args.revision))
    text = make_notes(args.milestone, args.project, args.revision)
    save_notes(text, args.file_name)
    return 0


if __name__ == '__main__':
    sys.exit(main(sys.argv))
